﻿using System;
using System.Collections.Generic;

namespace VCommon.VApplication.Authorization
{
    public class PermissionNode
    {
        private readonly PermissionTree _tree;
        private readonly List<PermissionNode> _children = new List<PermissionNode>();

        /// <summary>
        /// 权限代码, 产品内唯一
        /// </summary>
        public string Code { get; }

        /// <summary>
        /// 面向用户的显示名称
        /// </summary>
        public string Name { get; }

        /// <summary>
        /// 面向用户的描述
        /// </summary>
        public string Description { get; }

        /// <summary>
        /// 面向运营商的显示名称
        /// </summary>
        public string NameForOperator { get; }

        /// <summary>
        /// 面向运营商的描述
        /// </summary>
        public string DescriptionForOperator { get; }

        public MultiTenancySides TenancySide { get; }

        public IReadOnlyList<PermissionNode> Children => _children;

        internal PermissionNode(PermissionTree tree, string code, string name, string description, string nameForOperator, string descriptionForOperator, MultiTenancySides tenancySide = MultiTenancySides.Both)
        {
            _tree = tree;
            Code = code;
            Name = name;
            Description = description;
            NameForOperator = nameForOperator;
            DescriptionForOperator = descriptionForOperator;
            TenancySide = tenancySide;
        }

        public PermissionNode CreateChildNode(string code, string name, string description, string nameForOperator, string descriptionForOperator, MultiTenancySides tenancySide = MultiTenancySides.Both)
        {
            if (_tree.CodeHashSet.Contains(code)) throw new ArgumentException($"重复的权限代码:{code},名称:{name}");

            var node = new PermissionNode(_tree, code, name, description, nameForOperator, descriptionForOperator, tenancySide);
            _children.Add(node);
            _tree.CodeHashSet.Add(code);
            return node;
        }
    }
}