﻿using System;
using System.Collections.Generic;

namespace VCommon.VApplication.Authorization
{
    public class PermissionTree
    {
        private readonly List<PermissionNode> _roots = new List<PermissionNode>();

        private HashSet<string> _codeHashSet = new HashSet<string>();

        internal HashSet<string> CodeHashSet
        {
            get
            {
                if (null == _codeHashSet) throw new InvalidOperationException("Final called");
                return _codeHashSet;
            }
        }

        public IReadOnlyCollection<PermissionNode> Roots => _roots;

        public PermissionNode CreateRootNode(string code, string name, string description, string nameForOperator, string descriptionForOperator)
        {
            if (CodeHashSet.Contains(code)) throw new ArgumentException($"重复的权限代码:{code},名称:{name}");
            var node = new PermissionNode(this, code, name, description, nameForOperator, descriptionForOperator);
            _roots.Add(node);
            CodeHashSet.Add(code);
            return node;
        }

        public IReadOnlyCollection<PermissionNodeOutput> Filter(IReadOnlyCollection<string> licPerms)
        {
            return FltTree(_roots, new HashSet<string>(licPerms));
        }

        private static IReadOnlyCollection<PermissionNodeOutput> FltTree(IReadOnlyCollection<PermissionNode> nodesFrom, HashSet<string> licPerms, List<PermissionNodeOutput> output = null)
        {
            if (output == null) output = new List<PermissionNodeOutput>();

            foreach (var node in nodesFrom)
            {
                if (!licPerms.Contains(node.Code)) continue;

                var copy = new PermissionNodeOutput(node.Code, node.Name, node.Description, node.NameForOperator, node.DescriptionForOperator);
                output.Add(copy);

                FltTree(node.Children, licPerms, copy.Children);
            }

            return output;
        }

        /// <summary> 初始化完成, 释放并返回重名检查集合, 禁止再加挂节点 </summary>
        public HashSet<string> CompleteAdding()
        {
            var toReturn = _codeHashSet;
            _codeHashSet = null;
            return toReturn;
        }
    }
}